@@ -20,7 +20,7 @@ from elftools.elf.constants import SH_FLAGS
 
 
 __author__ = 'Erik Moqvist'
-__version__ = '17.13.0'
+__version__ = '17.14.3'
 
 
 DEFAULT_WORD_SIZE_BITS = 8
@@ -1063,22 +1063,33 @@ class BinFile:
 
         self.execution_start_address = elffile.header['e_entry']
 
-        for segment in elffile.iter_segments():
-            if segment['p_type'] != 'PT_LOAD':
+        for section in elffile.iter_sections():
+            if section['sh_type'] == 'SHT_NOBITS':
                 continue
 
-            address = segment['p_paddr']
-            offset = segment['p_offset']
-            size = segment['p_filesz']
+            if (section['sh_flags'] & SH_FLAGS.SHF_ALLOC) == 0:
+                continue
+
+            address = section['sh_addr']
+            offset = section['sh_offset']
+            size = section['sh_size']
 
             if size == 0:
                 continue
 
-            self._segments.add(_Segment(address,
-                                        address + size,
-                                        data[offset:offset + size],
-                                        self.word_size_bytes),
-                               overwrite)
+            for segment in elffile.iter_segments():
+                if segment['p_type'] != 'PT_LOAD':
+                    continue
+
+                if segment.section_in_segment(section):
+                    v_addr = segment['p_vaddr']
+                    p_addr = segment['p_paddr']
+                    address = p_addr + address - v_addr
+                    self._segments.add(_Segment(address,
+                                                address + size,
+                                                data[offset:offset + size],
+                                                self.word_size_bytes),
+                                       overwrite)
 
     def add_file(self, filename, overwrite=False):
         """Open given file and add its data by guessing its format. The format
@@ -1946,6 +1957,13 @@ def _do_as_ti_txt(args):
         print(bf.as_ti_txt(), end='')
 
 
+def _do_as_verilog_vmem(args):
+    for binfile in args.binfile:
+        bf = BinFile()
+        bf.add_file(binfile)
+        print(bf.as_verilog_vmem(), end='')
+
+
 def _do_fill(args):
     with open(args.infile, 'r') as fin:
         data = fin.read()
@@ -2086,6 +2104,15 @@ def _main():
                            help='One or more binary format files.')
     subparser.set_defaults(func=_do_as_ti_txt)
 
+    # The 'as_verilog_vmem' subparser.
+    subparser = subparsers.add_parser(
+        'as_verilog_vmem',
+        description='Print given file(s) as Verilog VMEM.')
+    subparser.add_argument('binfile',
+                           nargs='+',
+                           help='One or more binary format files.')
+    subparser.set_defaults(func=_do_as_verilog_vmem)
+
     # The 'fill' subparser.
     subparser = subparsers.add_parser(
         'fill',
